@@ -5,7 +5,7 @@ from time import time
 import pdf2image
 import pytesseract
 import numpy as np
-from .engine import get_instruments_dict, cropImage, predictParts
+from .engine import get_instruments_dict, cropImage, predictParts, predict_parts, parse_pytesseract_data
 from PyPDF2 import PdfFileReader
 
 
@@ -28,6 +28,8 @@ class PdfPredictor():
         tessdata_dir=None,
         log_stream=sys.stdout,
         use_multiprocessing=False,
+        crop_to_top=False,
+        crop_to_left=True,
         ):
         self.instruments = get_instruments_dict(
             instruments=instruments,
@@ -41,6 +43,9 @@ class PdfPredictor():
         self.tessdata_dir = tessdata_dir
         self.log_stream = log_stream
         self.use_multiprocessing = use_multiprocessing
+        self.crop_to_top = crop_to_top
+        self.crop_to_left = crop_to_left
+        self.crop = crop_to_top or crop_to_left
     
     def log(self, *msg):
         if self.log_stream is None:
@@ -49,17 +54,22 @@ class PdfPredictor():
 
     def parts(self):
         lastPartName = ""
+        lastPartNumber = 0
         lastPartNamePage = 0
         lastInstruments = []
         pdfReader = PdfFileReader(BytesIO(self.pdf))
         for i in range(pdfReader.getNumPages()):
             self.log("page", i+1, "of", pdfReader.getNumPages())
             img = pdf2image.convert_from_bytes(self.pdf, dpi=200, first_page=i+1, last_page=i+1)[0]
+            if self.crop:
+                self.log("cropping...")
+                if self.crop_to_top:
+                    img = img.crop((0, 0, img.width, img.height//2))
+                if self.crop_to_left:
+                    img = img.crop((0, 0, img.width//2, img.height))
             img = np.array(img)
-            self.log("cropping...")
-            img = cropImage(img)
             self.log("detecting...")
-            config = "--user-words sheetmusicUploader/instrumentsToLookFor.txt --psm 11 --dpi 96 -l eng"
+            config = "--user-words sheetmusicUploader/instrumentsToLookFor.txt --psm 11 --dpi 200 -l eng"
             if self.use_lstm: config += " --oem 1"
             if self.tessdata_dir != None: config += " --tessdata-dir \""+self.tessdata_dir+"\""
             pytesseract_args = [img]
@@ -69,36 +79,39 @@ class PdfPredictor():
             }
             timer = SimpleTimer()
             if self.use_multiprocessing:
-                detectionData = multiprocessing.Pool().apply(
+                detection_data = multiprocessing.Pool().apply(
                     pytesseract.image_to_data,
                     pytesseract_args,
                     pytesseract_kwargs,
                 )
             else:
-                detectionData = pytesseract.image_to_data(*pytesseract_args, **pytesseract_kwargs)
+                detection_data = pytesseract.image_to_data(*pytesseract_args, **pytesseract_kwargs)
             self.log(f"done in {timer} seconds")
             self.log("predicting...")
             timer = SimpleTimer()
-            partNames, instrumentses = predictParts(detectionData, self.instruments, img.shape[1], img.shape[0])
+            detections = parse_pytesseract_data(detection_data)
+            parts = list(predict_parts(detections, self.instruments))
             self.log(f"done in {timer} seconds")
-            self.log("partNames:", partNames, "instrumentses:", instrumentses)
-            for j in range(len(partNames)):
-                self.log(j, lastPartName)
-                if lastPartName == partNames[j]:
+            self.log("parts:", [part for part in parts])
+            for j, (name, part_number, instruments) in enumerate(parts):
+                if lastPartName == name:
                     continue
                 if lastPartName:
                     yield {
                         "name": lastPartName,
-                        "instruments": lastInstruments,
+                        "partNumber": lastPartNumber,
+                        "instruments": instruments,
                         "fromPage": lastPartNamePage,
                         "toPage": i if j == 0 else i+1
                     }
-                lastPartName = partNames[j]
+                lastPartName = name
+                lastPartNumber = part_number
                 lastPartNamePage = i+1
-                lastInstruments = instrumentses[j]
+                lastInstruments = instruments
         if lastPartName:
             yield {
                 "name": lastPartName,
+                "partNumber": lastPartNumber,
                 "instruments": lastInstruments,
                 "fromPage": lastPartNamePage,
                 "toPage": pdfReader.getNumPages()
